@@ -1469,6 +1469,8 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 	uint32_t *binding_table;
 	drm_intel_bo *binding_table_bo, *surface_state_bo;
 
+	composite_op->src_filter =
+	    sampler_state_filter_from_picture(source_picture->filter);
 	if (composite_op->src_filter < 0) {
 		intel_debug_fallback(scrn, "Bad src filter 0x%x\n",
 				     source_picture->filter);
@@ -1572,8 +1574,6 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 	intel->render_dest = dest;
 	drm_intel_bo_unreference(composite_op->binding_table_bo);
 	composite_op->binding_table_bo = binding_table_bo;
-	composite_op->src_filter =
-	    sampler_state_filter_from_picture(source_picture->filter);
 
 	intel->scale_units[0][0] = source->drawable.width;
 	intel->scale_units[0][1] = source->drawable.height;
@@ -1589,7 +1589,7 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 		intel->transform[1] = mask_picture->transform;
 		intel->scale_units[1][0] = mask->drawable.width;
 		intel->scale_units[1][1] = mask->drawable.height;
-		composite_op->is_affine |=
+		composite_op->is_affine &=
 		    i830_transform_is_affine(intel->transform[1]);
 	}
 
@@ -1627,7 +1627,7 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 	}
 
 	if (!i965_composite_check_aperture(scrn)) {
-		intel_batch_flush(scrn, FALSE);
+		intel_batch_submit(scrn);
 		if (!i965_composite_check_aperture(scrn)) {
 			intel_debug_fallback(scrn,
 					     "Couldn't fit render operation "
@@ -1638,7 +1638,7 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 
 	if(i830_uxa_pixmap_is_dirty(source) ||
 	   (mask && i830_uxa_pixmap_is_dirty(mask)))
-		intel_batch_pipelined_flush(scrn);
+		intel_batch_emit_flush(scrn);
 
 	intel->needs_render_state_emit = TRUE;
 
@@ -1682,7 +1682,7 @@ i965_composite(PixmapPtr dest, int srcX, int srcY, int maskX, int maskY,
 	float src_x[3], src_y[3], src_w[3], mask_x[3], mask_y[3], mask_w[3];
 	int i;
 	drm_intel_bo *vb_bo;
-	float vb[18];
+	float vb[24]; /* 3 * (2 dst + 3 src + 3 mask) */
 	Bool is_affine = render_state->composite_op.is_affine;
 
 	if (is_affine) {
@@ -1801,11 +1801,10 @@ i965_composite(PixmapPtr dest, int srcX, int srcY, int maskX, int maskY,
 		if (!is_affine)
 			vb[i++] = mask_w[0];
 	}
-	assert(i <= VERTEX_BUFFER_SIZE);
 	drm_intel_bo_subdata(vb_bo, render_state->vb_offset * 4, i * 4, vb);
 
 	if (!i965_composite_check_aperture(scrn))
-		intel_batch_flush(scrn, FALSE);
+		intel_batch_submit(scrn);
 
 	intel_batch_start_atomic(scrn, 200);
 	if (intel->needs_render_state_emit)
